﻿#region License Information
/* HeuristicLab
 * Copyright (C) 2002-2012 Heuristic and Evolutionary Algorithms Laboratory (HEAL)
 *
 * This file is part of HeuristicLab.
 *
 * HeuristicLab is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * HeuristicLab is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with HeuristicLab. If not, see <http://www.gnu.org/licenses/>.
 */
#endregion

using HeuristicLab.Common;
using HeuristicLab.Core;
using HeuristicLab.Data;
using HeuristicLab.Persistence.Default.CompositeSerializers.Storable;

namespace HeuristicLab.Encodings.CombinedIntegerVectorEncoding {
  /// <summary>
  /// Generates a new random combined integer vector with each element uniformly distributed in a specified range.
  /// </summary>
  [Item("UniformRandomCombinedIntegerVectorCreator", "An operator which creates a new random combined int vector with each element uniformly distributed in a specified range.")]
  [StorableClass]
  public class UniformRandomCombinedIntegerVectorCreator : CombinedIntegerVectorCreator {
    [StorableConstructor]
    protected UniformRandomCombinedIntegerVectorCreator(bool deserializing) : base(deserializing) { }
    protected UniformRandomCombinedIntegerVectorCreator(UniformRandomCombinedIntegerVectorCreator original, Cloner cloner) : base(original, cloner) { }
    public UniformRandomCombinedIntegerVectorCreator() : base() { }

    public override IDeepCloneable Clone(Cloner cloner) {
      return new UniformRandomCombinedIntegerVectorCreator(this, cloner);
    }

    /// <summary>
    /// Generates a new random integer vector with the given <paramref name="length"/>.
    /// </summary>
    /// <param name="random">The random number generator.</param>
    /// <param name="length">The length of the int vector.</param>
    /// <param name="bounds">A matrix containing the inclusive lower and inclusive upper bound in the first and second column and a step size in the third column.
    /// Each line represents the bounds for a certain dimension. If fewer lines are given, the lines are cycled.</param>
    /// <returns>The newly created integer vector.</returns>
    public static CombinedIntegerVector Apply(IRandom random, int length, IntMatrix bounds, int actionPartLength) {
      var result = new CombinedIntegerVector(length, actionPartLength, GetElementsOfBounds(bounds));
      result.Randomize(random, bounds);
      return result;
    }

    private static int[,] GetElementsOfBounds(IntMatrix bounds) {
      int[,] realBounds = new int[bounds.Rows, bounds.Columns];
      for (int i = 0; i < bounds.Rows; i++) {
        for (int j = 0; j < bounds.Columns; j++) {
          realBounds[i, j] = bounds[i, j];
        }
      }
      return realBounds;
    }

    /// <summary>
    /// Forwards the call to <see cref="Apply(IRandom, int, int, int)"/>.
    /// </summary>
    /// <param name="random">The pseudo random number generator to use.</param>
    /// <param name="length">The length of the int vector.</param>
    /// <param name="bounds">Contains in each row for each dimension minimum (inclusive), maximum (inclusive), and step size.</param>
    /// <returns>The newly created int vector.</returns>
    protected override CombinedIntegerVector Create(IRandom random, IntValue length, IntMatrix bounds, IntValue actionPartLength) {
      return Apply(random, length.Value, bounds, actionPartLength.Value);
    }
  }
}